/*
 * tut9.c -- SDL-Tutorial, 9. Kapitel
 * NO Copyright (c) by Nicolai 'Prefect' Haehnle
 *
 * Diese Software ist Public Domain.
 */

#include <stdlib.h>
#include <stdio.h>

#include <SDL.h>
#include <SDL_opengl.h>

unsigned g_texPicture;


unsigned LoadTexture(const char *filename)
{
	SDL_Surface *bitmap, *conv;
	unsigned texture;

/* Lade die Datei */
	bitmap = SDL_LoadBMP(filename);
	if (!bitmap) {
		fprintf(stderr, "%s: konnte nicht geladen werden\n", filename);
		exit(1);
	}

/* Konvertiere ins 32-Bit RGBA-Format */
	conv = SDL_CreateRGBSurface(SDL_SWSURFACE, bitmap->w, bitmap->h, 32,
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
			0xff000000, 0x00ff0000, 0x0000ff00, 0x000000ff);
#else
                        0x000000ff, 0x0000ff00, 0x00ff0000, 0xff000000);
#endif
	SDL_BlitSurface(bitmap, 0, conv, 0);

/* Erstelle eine OpenGL-Textur */
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
        glPixelStorei(GL_UNPACK_ROW_LENGTH, conv->pitch / conv->format->BytesPerPixel);
	glTexImage2D(GL_TEXTURE_2D, 0, 3, conv->w, conv->h, 0, GL_RGBA,
		GL_UNSIGNED_BYTE, conv->pixels);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 0);

/* Gib die verwendeten Surfaces und Speicherplatz frei */
	SDL_FreeSurface(bitmap);
	SDL_FreeSurface(conv);

	return texture;
}


void RefreshScreen()
{
	glColor3ub(255, 255, 255);

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, g_texPicture);

	glBegin(GL_QUADS);
		glTexCoord2f(0, 0);	glVertex2i(0, 0);
		glTexCoord2f(1, 0);	glVertex2i(640, 0);
		glTexCoord2f(1, 1);	glVertex2i(640, 480);
		glTexCoord2f(0, 1);	glVertex2i(0, 480);
	glEnd();
}

int main(int argc, char **argv)
{
	SDL_Surface *screen;
	int running;

/* SDL initialisieren */
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		fprintf(stderr, "SDL konnte nicht initialisiert werden: %s\n",
			SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

/* OpenGL-Bildschirmmodus */
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);

	screen = SDL_SetVideoMode(640, 480, 0, SDL_OPENGL);
	if (!screen) {
		fprintf(stderr, "Konnte Bildschirmmodus nicht setzen: %s\n",
			SDL_GetError());
		exit(1);
	}

/* Lade Texturen */
	g_texPicture = LoadTexture("galaxien.bmp");

/* Initialisierung von OpenGL */
	glViewport(0, 0, screen->w, screen->h);

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(0, screen->w, screen->h, 0, -1.0, 1.0);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

/* Programmschleife */
	running = 1;
	while(running) {
		SDL_Event event;

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				running = 0;
				break;
			case SDL_QUIT:
				running = 0;
				break;
			}
		}


		RefreshScreen();

		// Pageflip: Back- und Frontbuffer werden vertauscht
		SDL_GL_SwapBuffers();
	}

/* Gib die OpenGL-Texturen frei */
	glDeleteTextures(1, &g_texPicture);

        return 0;
}
